@@ -52,9 +52,11 @@ enum {
 	PROP_READ_ONLY,
 	PROP_ACCEPT_POLICY,
 
-	LAST_PROP
+	LAST_PROPERTY
 };
 
+static GParamSpec *properties[LAST_PROPERTY] = { NULL, };
+
 typedef struct {
 	gboolean constructed, read_only;
 	GHashTable *domains, *serials;
@@ -190,14 +192,13 @@ soup_cookie_jar_class_init (SoupCookieJarClass *jar_class)
 			      SOUP_TYPE_COOKIE | G_SIGNAL_TYPE_STATIC_SCOPE,
 			      SOUP_TYPE_COOKIE | G_SIGNAL_TYPE_STATIC_SCOPE);
 
-	g_object_class_install_property (
-		object_class, PROP_READ_ONLY,
+        properties[PROP_READ_ONLY] =
 		g_param_spec_boolean ("read-only",
 				      "Read-only",
 				      "Whether or not the cookie jar is read-only",
 				      FALSE,
 				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-				      G_PARAM_STATIC_STRINGS));
+				      G_PARAM_STATIC_STRINGS);
 
 	/**
 	 * SoupCookieJar:accept-policy:
@@ -205,15 +206,16 @@ soup_cookie_jar_class_init (SoupCookieJarClass *jar_class)
 	 * The policy the jar should follow to accept or reject cookies
 	 *
 	 */
-	g_object_class_install_property (
-		object_class, PROP_ACCEPT_POLICY,
+        properties[PROP_ACCEPT_POLICY] =
 		g_param_spec_enum ("accept-policy",
 				   "Accept-policy",
 				   "The policy the jar should follow to accept or reject cookies",
 				   SOUP_TYPE_COOKIE_JAR_ACCEPT_POLICY,
 				   SOUP_COOKIE_JAR_ACCEPT_ALWAYS,
 				   G_PARAM_READWRITE |
-				   G_PARAM_STATIC_STRINGS));
+				   G_PARAM_STATIC_STRINGS);
+
+        g_object_class_install_properties (object_class, LAST_PROPERTY, properties);
 }
 
 /**
@@ -1000,7 +1002,7 @@ soup_cookie_jar_set_accept_policy (SoupCookieJar *jar,
 
 	if (priv->accept_policy != policy) {
 		priv->accept_policy = policy;
-		g_object_notify (G_OBJECT (jar), "accept-policy");
+		g_object_notify_by_pspec (G_OBJECT (jar), properties[PROP_ACCEPT_POLICY]);
 	}
 }
 
