@@ -13,6 +13,7 @@
 #include "soup.h"
 #include "soup-io-stream.h"
 #include "soup-message-queue-item.h"
+#include "soup-client-message-io-http1.h"
 #include "soup-socket-properties.h"
 #include "soup-private-enum-types.h"
 #include <gio/gnetworking.h>
@@ -27,11 +28,13 @@ typedef struct {
 	GIOStream *iostream;
 	SoupSocketProperties *socket_props;
         guint64 id;
+        GSocketAddress *remote_address;
 
 	GUri *proxy_uri;
 	gboolean ssl;
 
 	SoupMessage *current_msg;
+        SoupClientMessageIO *io_data;
 	SoupConnectionState state;
 	time_t       unused_timeout;
 	GSource     *idle_timeout_src;
@@ -56,15 +59,18 @@ enum {
 
         PROP_ID,
 	PROP_REMOTE_CONNECTABLE,
+        PROP_REMOTE_ADDRESS,
 	PROP_SOCKET_PROPERTIES,
 	PROP_STATE,
 	PROP_SSL,
 	PROP_TLS_CERTIFICATE,
 	PROP_TLS_CERTIFICATE_ERRORS,
 
-	LAST_PROP
+	LAST_PROPERTY
 };
 
+static GParamSpec *properties[LAST_PROPERTY] = { NULL, };
+
 static void stop_idle_timer (SoupConnectionPrivate *priv);
 
 /* Number of seconds after which we close a connection that hasn't yet
@@ -84,7 +90,9 @@ soup_connection_finalize (GObject *object)
 
 	g_clear_pointer (&priv->proxy_uri, g_uri_unref);
 	g_clear_pointer (&priv->socket_props, soup_socket_properties_unref);
+        g_clear_pointer (&priv->io_data, soup_client_message_io_destroy);
 	g_clear_object (&priv->remote_connectable);
+        g_clear_object (&priv->remote_address);
 	g_clear_object (&priv->current_msg);
 
 	if (priv->cancellable) {
@@ -152,6 +160,9 @@ soup_connection_get_property (GObject *object, guint prop_id,
 	case PROP_REMOTE_CONNECTABLE:
 		g_value_set_object (value, priv->remote_connectable);
 		break;
+        case PROP_REMOTE_ADDRESS:
+                g_value_set_object (value, priv->remote_address);
+	        break;
 	case PROP_SOCKET_PROPERTIES:
 		g_value_set_boxed (value, priv->socket_props);
 		break;
@@ -218,61 +229,63 @@ soup_connection_class_init (SoupConnectionClass *connection_class)
 			      G_TYPE_NONE, 0);
 
 	/* properties */
-	g_object_class_install_property (
-                object_class, PROP_REMOTE_CONNECTABLE,
+        properties[PROP_REMOTE_CONNECTABLE] =
                 g_param_spec_object ("remote-connectable",
                                      "Remote Connectable",
                                      "Socket to connect to make outgoing connections on",
                                      G_TYPE_SOCKET_CONNECTABLE,
                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-                                     G_PARAM_STATIC_STRINGS));
-	g_object_class_install_property (
-		object_class, PROP_SOCKET_PROPERTIES,
+                                     G_PARAM_STATIC_STRINGS);
+        properties[PROP_REMOTE_ADDRESS] =
+                g_param_spec_object ("remote-address",
+                                     "Remote Address",
+                                     "Remote address of connection",
+                                     G_TYPE_SOCKET_ADDRESS,
+                                     G_PARAM_READABLE |
+                                     G_PARAM_STATIC_STRINGS);
+        properties[PROP_SOCKET_PROPERTIES] =
 		g_param_spec_boxed ("socket-properties",
 				    "Socket properties",
 				    "Socket properties",
 				    SOUP_TYPE_SOCKET_PROPERTIES,
 				    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-				    G_PARAM_STATIC_STRINGS));
-	g_object_class_install_property (
-		object_class, PROP_STATE,
+				    G_PARAM_STATIC_STRINGS);
+        properties[PROP_STATE] =
 		g_param_spec_enum ("state",
 				   "Connection state",
 				   "Current state of connection",
 				   SOUP_TYPE_CONNECTION_STATE, SOUP_CONNECTION_NEW,
 				   G_PARAM_READWRITE |
-				   G_PARAM_STATIC_STRINGS));
-	g_object_class_install_property (
-		object_class, PROP_SSL,
+				   G_PARAM_STATIC_STRINGS);
+        properties[PROP_SSL] =
 		g_param_spec_boolean ("ssl",
 				      "Connection uses TLS",
 				      "Whether the connection should use TLS",
 				      FALSE,G_PARAM_READWRITE |
-				      G_PARAM_STATIC_STRINGS));
-	g_object_class_install_property (
-		object_class, PROP_ID,
+				      G_PARAM_STATIC_STRINGS);
+        properties[PROP_ID] =
 		g_param_spec_uint64 ("id",
                                      "Connection Identifier",
                                      "Unique identifier for the connection",
                                      0, G_MAXUINT64,
                                      0, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-                                     G_PARAM_STATIC_STRINGS));
-	g_object_class_install_property (
-                object_class, PROP_TLS_CERTIFICATE,
+                                     G_PARAM_STATIC_STRINGS);
+        properties[PROP_TLS_CERTIFICATE] =
 		g_param_spec_object ("tls-certificate",
                                      "TLS Certificate",
                                      "The TLS certificate associated with the connection",
                                      G_TYPE_TLS_CERTIFICATE,
                                      G_PARAM_READABLE |
-	                             G_PARAM_STATIC_STRINGS));
-	g_object_class_install_property (
-                object_class, PROP_TLS_CERTIFICATE_ERRORS,
+	                             G_PARAM_STATIC_STRINGS);
+        properties[PROP_TLS_CERTIFICATE_ERRORS] =
                 g_param_spec_flags ("tls-certificate-errors",
                                     "TLS Certificate Errors",
                                     "The verification errors on the connections's TLS certificate",
                                     G_TYPE_TLS_CERTIFICATE_FLAGS, 0,
                                     G_PARAM_READABLE |
-                                    G_PARAM_STATIC_STRINGS));
+                                    G_PARAM_STATIC_STRINGS);
+
+        g_object_class_install_properties (object_class, LAST_PROPERTY, properties);
 }
 
 static void
@@ -444,7 +457,7 @@ tls_connection_accept_certificate (SoupConnection      *conn,
 static void
 tls_connection_peer_certificate_changed (SoupConnection *conn)
 {
-	g_object_notify (G_OBJECT (conn), "tls-certificate");
+	g_object_notify_by_pspec (G_OBJECT (conn), properties[PROP_TLS_CERTIFICATE]);
 }
 
 static GTlsClientConnection *
@@ -485,15 +498,17 @@ soup_connection_connected (SoupConnection    *conn,
 {
         SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
         GSocket *socket;
-        GSocketAddress *addr;
 
         socket = g_socket_connection_get_socket (connection);
         g_socket_set_timeout (socket, priv->socket_props->io_timeout);
         g_socket_set_option (socket, IPPROTO_TCP, TCP_NODELAY, TRUE, NULL);
 
-        addr = g_socket_get_remote_address (socket, NULL);
-        if (addr && G_IS_PROXY_ADDRESS (addr)) {
-                GProxyAddress *paddr = G_PROXY_ADDRESS (addr);
+        g_clear_object (&priv->remote_address);
+        priv->remote_address = g_socket_get_remote_address (socket, NULL);
+        g_object_notify_by_pspec (G_OBJECT (conn), properties[PROP_REMOTE_ADDRESS]);
+
+        if (priv->remote_address && G_IS_PROXY_ADDRESS (priv->remote_address)) {
+                GProxyAddress *paddr = G_PROXY_ADDRESS (priv->remote_address);
 
                 if (strcmp (g_proxy_address_get_protocol (paddr), "http") == 0) {
                         GError *error = NULL;
@@ -504,7 +519,6 @@ soup_connection_connected (SoupConnection    *conn,
                         }
                 }
         }
-        g_clear_object (&addr);
 
         if (priv->ssl && !priv->proxy_uri) {
                 GTlsClientConnection *tls_connection;
@@ -893,6 +907,9 @@ soup_connection_steal_iostream (SoupConnection *conn)
                                 g_object_ref (socket), g_object_unref);
         g_clear_object (&priv->connection);
 
+        if (priv->io_data)
+                soup_client_message_io_stolen (priv->io_data);
+
         return iostream;
 }
 
@@ -979,6 +996,7 @@ soup_connection_set_state (SoupConnection *conn, SoupConnectionState state)
 	g_return_if_fail (state >= SOUP_CONNECTION_NEW &&
 			  state <= SOUP_CONNECTION_DISCONNECTED);
 
+        g_object_ref (conn);
 	g_object_freeze_notify (G_OBJECT (conn));
 
 	priv = soup_connection_get_instance_private (conn);
@@ -998,10 +1016,11 @@ soup_connection_set_state (SoupConnection *conn, SoupConnectionState state)
 		if (priv->state == SOUP_CONNECTION_IDLE)
 			start_idle_timer (conn);
 
-		g_object_notify (G_OBJECT (conn), "state");
+		g_object_notify_by_pspec (G_OBJECT (conn), properties[PROP_STATE]);
 	}
 
 	g_object_thaw_notify (G_OBJECT (conn));
+        g_object_unref (conn);
 }
 
 void
@@ -1025,26 +1044,34 @@ soup_connection_get_ever_used (SoupConnection *conn)
 	return priv->unused_timeout == 0;
 }
 
-void
-soup_connection_send_request (SoupConnection           *conn,
-			      SoupMessageQueueItem     *item,
-			      SoupMessageIOCompletionFn completion_cb,
-			      gpointer                  user_data)
+SoupClientMessageIO *
+soup_connection_setup_message_io (SoupConnection *conn,
+                                  SoupMessage    *msg)
 {
-	SoupConnectionPrivate *priv;
+        SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
 
-	g_return_if_fail (SOUP_IS_CONNECTION (conn));
-	g_return_if_fail (item != NULL);
-	priv = soup_connection_get_instance_private (conn);
-	g_return_if_fail (priv->state != SOUP_CONNECTION_NEW &&
-			  priv->state != SOUP_CONNECTION_DISCONNECTED);
+        g_assert (priv->state != SOUP_CONNECTION_NEW &&
+                  priv->state != SOUP_CONNECTION_DISCONNECTED);
 
-	if (item->msg != priv->current_msg)
-		set_current_msg (conn, item->msg);
-	else
-		priv->reusable = FALSE;
+        if (priv->current_msg != msg)
+                set_current_msg (conn, msg);
+        else
+                priv->reusable = FALSE;
 
-	soup_message_send_request (item, completion_cb, user_data);
+        g_assert (priv->io_data == NULL);
+        priv->io_data = soup_client_message_io_http1_new (priv->iostream);
+
+        return priv->io_data;
+}
+
+void
+soup_connection_message_io_finished (SoupConnection *conn,
+                                     SoupMessage    *msg)
+{
+        SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
+
+        g_assert (priv->current_msg == msg);
+        g_clear_pointer (&priv->io_data, soup_client_message_io_destroy);
 }
 
 GTlsCertificate *
@@ -1084,3 +1111,11 @@ soup_connection_get_id (SoupConnection *conn)
 
         return priv->id;
 }
+
+GSocketAddress *
+soup_connection_get_remote_address (SoupConnection *conn)
+{
+        SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
+
+        return priv->remote_address;
+}
