@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 /*
  * soup-session.c
  *
@@ -10,7 +10,6 @@
 #endif
 
 #include <glib/gi18n-lib.h>
-#include <gmodule.h>
 #include "gconstructor.h"
 
 #ifdef G_OS_WIN32
@@ -20,21 +19,6 @@
 HMODULE soup_dll;
 #endif
 
-static gboolean
-soup3_is_loaded (void)
-{
-    GModule *module = g_module_open (NULL, 0);
-    gpointer func;
-    gboolean result = FALSE;
-
-    if (g_module_symbol (module, "soup_date_time_new_from_http_string", &func))
-        result = TRUE;
-
-    g_module_close (module);
-
-    return result;
-}
-
 static void
 soup_init (void)
 {
@@ -50,9 +34,6 @@ soup_init (void)
 #ifdef HAVE_BIND_TEXTDOMAIN_CODESET
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 #endif
-
-        if (soup3_is_loaded ())
-                g_error ("libsoup3 symbols detected. Using libsoup2 and libsoup3 in the same process is not supported.");
 }
 
 #if defined (G_OS_WIN32)
