@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 /*
  * soup-message-headers.c: HTTP message header arrays
  *
@@ -58,8 +58,6 @@ struct _SoupMessageHeaders {
 	goffset content_length;
 	SoupExpectation expectations;
 	char *content_type;
-
-	gatomicrefcount ref_count;
 };
 
 /**
@@ -77,43 +75,54 @@ soup_message_headers_new (SoupMessageHeadersType type)
 {
 	SoupMessageHeaders *hdrs;
 
-	hdrs = g_slice_new0 (SoupMessageHeaders);
+	hdrs = g_atomic_rc_box_new0 (SoupMessageHeaders);
 	/* FIXME: is "5" a good default? */
 	hdrs->array = g_array_sized_new (TRUE, FALSE, sizeof (SoupHeader), 5);
 	hdrs->type = type;
 	hdrs->encoding = -1;
 
-        g_atomic_ref_count_init (&hdrs->ref_count);
-
-	return hdrs;
-}
-
-static SoupMessageHeaders *
-soup_message_headers_copy (SoupMessageHeaders *hdrs)
-{
-	g_atomic_ref_count_inc (&hdrs->ref_count);
 	return hdrs;
 }
 
 /**
- * soup_message_headers_free:
+ * soup_message_headers_ref:
  * @hdrs: a #SoupMessageHeaders
  *
- * Frees @hdrs.
- **/
-void
-soup_message_headers_free (SoupMessageHeaders *hdrs)
+ * Atomically increments the reference count of @hdrs by one.
+ *
+ * Returns: the passed in #SoupMessageHeaders
+ */
+SoupMessageHeaders *
+soup_message_headers_ref (SoupMessageHeaders *hdrs)
 {
-	if (!g_atomic_ref_count_dec (&hdrs->ref_count))
-		return;
+	g_atomic_rc_box_acquire (hdrs);
 
-	soup_message_headers_clear (hdrs);
-	g_array_free (hdrs->array, TRUE);
-	g_clear_pointer (&hdrs->concat, g_hash_table_destroy);
-	g_slice_free (SoupMessageHeaders, hdrs);
+	return hdrs;
 }
 
-G_DEFINE_BOXED_TYPE (SoupMessageHeaders, soup_message_headers, soup_message_headers_copy, soup_message_headers_free)
+static void
+soup_message_headers_destroy (SoupMessageHeaders *hdrs)
+{
+        soup_message_headers_clear (hdrs);
+        g_array_free (hdrs->array, TRUE);
+        g_clear_pointer (&hdrs->concat, g_hash_table_destroy);
+}
+
+/**
+ * soup_message_headers_unref:
+ * @hdrs: a #SoupMessageHeaders
+ *
+ * Atomically decrements the reference count of @hdrs by one.
+ * When the reference count reaches zero, the resources allocated by
+ * @hdrs are freed
+ */
+void
+soup_message_headers_unref (SoupMessageHeaders *hdrs)
+{
+        g_atomic_rc_box_release_full (hdrs, (GDestroyNotify)soup_message_headers_destroy);
+}
+
+G_DEFINE_BOXED_TYPE (SoupMessageHeaders, soup_message_headers, soup_message_headers_ref, soup_message_headers_unref)
 
 /**
  * soup_message_headers_get_headers_type:
