@@ -8,6 +8,8 @@
 
 #include "soup-filter-input-stream.h"
 #include "soup-message-headers.h"
+#include "soup-message-io-source.h"
+#include "soup-message-io-completion.h"
 
 typedef enum {
 	SOUP_MESSAGE_IO_STATE_NOT_STARTED,
@@ -31,16 +33,6 @@ typedef enum {
 	(SOUP_MESSAGE_IO_STATE_ACTIVE (state) && \
 	 state != SOUP_MESSAGE_IO_STATE_BODY_DONE)
 
-typedef enum {
-	SOUP_MESSAGE_IO_COMPLETE,
-        SOUP_MESSAGE_IO_INTERRUPTED,
-        SOUP_MESSAGE_IO_STOLEN
-} SoupMessageIOCompletion;
-
-typedef void (*SoupMessageIOCompletionFn) (GObject                *msg,
-					   SoupMessageIOCompletion completion,
-					   gpointer                user_data);
-
 typedef struct {
 	GIOStream              *iostream;
 	SoupFilterInputStream  *istream;
@@ -83,9 +75,6 @@ gboolean soup_message_io_data_read_headers (SoupMessageIOData *io,
                                             gushort           *extra_bytes,
 					    GError           **error);
 
-typedef gboolean (*SoupMessageIOSourceFunc) (GObject *msg,
-					     gpointer user_data);
-
 GSource *soup_message_io_data_get_source   (SoupMessageIOData      *io,
 					    GObject                *msg,
 					    GCancellable           *cancellable,
