@@ -8,15 +8,12 @@
 
 #include "soup-filter-input-stream.h"
 #include "soup-message.h"
-#include "soup-message-io-data.h"
+#include "soup-client-message-io.h"
 #include "auth/soup-auth.h"
 #include "soup-content-processor.h"
 #include "content-sniffer/soup-content-sniffer.h"
 #include "soup-session.h"
 
-typedef struct _SoupClientMessageIOData SoupClientMessageIOData;
-void soup_client_message_io_data_free (SoupClientMessageIOData *io);
-
 void             soup_message_set_status       (SoupMessage      *msg,
 						guint             status_code,
 						const char       *reason_phrase);
@@ -33,9 +30,10 @@ typedef guint    (*SoupMessageParseHeadersFn)(SoupMessage      *msg,
 					      gpointer          user_data,
 					      GError          **error);
 
-void soup_message_send_request (SoupMessageQueueItem      *item,
-				SoupMessageIOCompletionFn  completion_cb,
-				gpointer                   user_data);
+void soup_message_send_item (SoupMessage              *msg,
+                             SoupMessageQueueItem     *item,
+                             SoupMessageIOCompletionFn completion_cb,
+                             gpointer                  user_data);
 
 /* Auth handling */
 void           soup_message_set_auth       (SoupMessage *msg,
@@ -50,19 +48,10 @@ GUri          *soup_message_get_uri_for_auth (SoupMessage *msg);
 void       soup_message_io_run         (SoupMessage *msg,
 					gboolean     blocking);
 void       soup_message_io_finished    (SoupMessage *msg);
-void       soup_message_io_cleanup     (SoupMessage *msg);
 void       soup_message_io_pause       (SoupMessage *msg);
 void       soup_message_io_unpause     (SoupMessage *msg);
 gboolean   soup_message_is_io_paused   (SoupMessage *msg);
 gboolean   soup_message_io_in_progress (SoupMessage *msg);
-void       soup_message_io_stolen      (SoupMessage *msg);
-
-gboolean soup_message_io_read_headers          (SoupMessage           *msg,
-                                                SoupFilterInputStream *stream,
-                                                GByteArray            *buffer,
-                                                gboolean               blocking,
-                                                GCancellable          *cancellable,
-                                                GError               **error);
 
 gboolean soup_message_io_run_until_finish      (SoupMessage        *msg,
                                                 gboolean            blocking,
@@ -81,12 +70,6 @@ gboolean soup_message_io_run_until_read_finish (SoupMessage        *msg,
                                                 GAsyncResult       *result,
                                                 GError            **error);
 
-typedef gboolean (*SoupMessageSourceFunc) (SoupMessage *, gpointer);
-GSource *soup_message_io_get_source       (SoupMessage           *msg,
-					   GCancellable          *cancellable,
-					   SoupMessageSourceFunc  callback,
-					   gpointer               user_data);
-
 GInputStream *soup_message_io_get_response_istream (SoupMessage  *msg,
 						    GError      **error);
 
@@ -122,9 +105,7 @@ SoupConnection *soup_message_get_connection (SoupMessage    *msg);
 void            soup_message_set_connection (SoupMessage    *msg,
 					     SoupConnection *conn);
 
-SoupClientMessageIOData *soup_message_get_io_data (SoupMessage             *msg);
-void                     soup_message_set_io_data (SoupMessage             *msg,
-						   SoupClientMessageIOData *io);
+SoupClientMessageIO *soup_message_get_io_data (SoupMessage             *msg);
 
 SoupContentSniffer *soup_message_get_content_sniffer    (SoupMessage        *msg);
 void                soup_message_set_content_sniffer    (SoupMessage        *msg,
