@@ -59,7 +59,7 @@ struct _SoupMessage {
 };
 
 typedef struct {
-	SoupClientMessageIOData *io_data;
+	SoupClientMessageIO *io_data;
 
         SoupMessageHeaders *request_headers;
 	SoupMessageHeaders *response_headers;
@@ -94,6 +94,7 @@ typedef struct {
 	gboolean is_top_level_navigation;
         gboolean is_options_ping;
         guint    last_connection_id;
+        GSocketAddress *remote_address;
 
         SoupMessageMetrics *metrics;
 } SoupMessagePrivate;
@@ -138,14 +139,17 @@ enum {
 	PROP_RESPONSE_HEADERS,
 	PROP_TLS_PEER_CERTIFICATE,
 	PROP_TLS_PEER_CERTIFICATE_ERRORS,
+        PROP_REMOTE_ADDRESS,
 	PROP_PRIORITY,
 	PROP_SITE_FOR_COOKIES,
 	PROP_IS_TOP_LEVEL_NAVIGATION,
         PROP_IS_OPTIONS_PING,
 
-	LAST_PROP
+	LAST_PROPERTY
 };
 
+static GParamSpec *properties[LAST_PROPERTY] = { NULL, };
+
 static void
 soup_message_init (SoupMessage *msg)
 {
@@ -166,8 +170,6 @@ soup_message_finalize (GObject *object)
 
 	soup_message_set_connection (msg, NULL);
 
-	soup_client_message_io_data_free (priv->io_data);
-
 	g_clear_pointer (&priv->uri, g_uri_unref);
 	g_clear_pointer (&priv->first_party, g_uri_unref);
 	g_clear_pointer (&priv->site_for_cookies, g_uri_unref);
@@ -179,6 +181,7 @@ soup_message_finalize (GObject *object)
 	g_clear_pointer (&priv->disabled_features, g_hash_table_destroy);
 
 	g_clear_object (&priv->tls_peer_certificate);
+        g_clear_object (&priv->remote_address);
 
 	soup_message_headers_unref (priv->request_headers);
 	soup_message_headers_unref (priv->response_headers);
@@ -276,6 +279,9 @@ soup_message_get_property (GObject *object, guint prop_id,
 	case PROP_TLS_PEER_CERTIFICATE_ERRORS:
 		g_value_set_flags (value, priv->tls_peer_certificate_errors);
 		break;
+        case PROP_REMOTE_ADDRESS:
+                g_value_set_object (value, priv->remote_address);
+                break;
 	case PROP_PRIORITY:
 		g_value_set_enum (value, priv->priority);
 		break;
@@ -605,56 +611,50 @@ soup_message_class_init (SoupMessageClass *message_class)
 			      G_TYPE_NONE, 0);
 
 	/* properties */
-	g_object_class_install_property (
-		object_class, PROP_METHOD,
+        properties[PROP_METHOD] =
 		g_param_spec_string ("method",
 				     "Method",
 				     "The message's HTTP method",
 				     SOUP_METHOD_GET,
 				     G_PARAM_READWRITE |
-				     G_PARAM_STATIC_STRINGS));
-	g_object_class_install_property (
-		object_class, PROP_URI,
+				     G_PARAM_STATIC_STRINGS);
+        properties[PROP_URI] =
 		g_param_spec_boxed ("uri",
 				    "URI",
 				    "The message's Request-URI",
 				    G_TYPE_URI,
 				    G_PARAM_READWRITE |
-				    G_PARAM_STATIC_STRINGS));
-	g_object_class_install_property (
-		object_class, PROP_HTTP_VERSION,
+				    G_PARAM_STATIC_STRINGS);
+        properties[PROP_HTTP_VERSION] =
 		g_param_spec_enum ("http-version",
 				   "HTTP Version",
 				   "The HTTP protocol version to use",
 				   SOUP_TYPE_HTTP_VERSION,
 				   SOUP_HTTP_1_1,
 				   G_PARAM_READWRITE |
-				   G_PARAM_STATIC_STRINGS));
-	g_object_class_install_property (
-		object_class, PROP_FLAGS,
+				   G_PARAM_STATIC_STRINGS);
+        properties[PROP_FLAGS] =
 		g_param_spec_flags ("flags",
 				    "Flags",
 				    "Various message options",
 				    SOUP_TYPE_MESSAGE_FLAGS,
 				    0,
 				    G_PARAM_READWRITE |
-				    G_PARAM_STATIC_STRINGS));
-	g_object_class_install_property (
-		object_class, PROP_STATUS_CODE,
+				    G_PARAM_STATIC_STRINGS);
+        properties[PROP_STATUS_CODE] =
 		g_param_spec_uint ("status-code",
 				   "Status code",
 				   "The HTTP response status code",
 				   0, 999, 0,
 				   G_PARAM_READABLE |
-				   G_PARAM_STATIC_STRINGS));
-	g_object_class_install_property (
-		object_class, PROP_REASON_PHRASE,
+				   G_PARAM_STATIC_STRINGS);
+        properties[PROP_REASON_PHRASE] =
 		g_param_spec_string ("reason-phrase",
 				     "Reason phrase",
 				     "The HTTP response reason phrase",
 				     NULL,
 				     G_PARAM_READABLE |
-				     G_PARAM_STATIC_STRINGS));
+				     G_PARAM_STATIC_STRINGS);
 	/**
 	 * SoupMessage:first-party:
 	 *
@@ -662,84 +662,90 @@ soup_message_class_init (SoupMessageClass *message_class)
 	 * queued.
 	 *
 	 */
-	g_object_class_install_property (
-		object_class, PROP_FIRST_PARTY,
+        properties[PROP_FIRST_PARTY] =
 		g_param_spec_boxed ("first-party",
 				    "First party",
 				    "The URI loaded in the application when the message was requested.",
 				    G_TYPE_URI,
 				    G_PARAM_READWRITE |
-				    G_PARAM_STATIC_STRINGS));
+				    G_PARAM_STATIC_STRINGS);
 	/**
 	 * SoupMessage:site-for-cookkies:
 	 *
 	 * Site used to compare cookies against. Used for SameSite cookie support.
 	 *
 	 */
-	g_object_class_install_property (
-		object_class, PROP_SITE_FOR_COOKIES,
+        properties[PROP_SITE_FOR_COOKIES] =
 		g_param_spec_boxed ("site-for-cookies",
 				    "Site for cookies",
 				    "The URI for the site to compare cookies against",
 				    G_TYPE_URI,
-				    G_PARAM_READWRITE));
+				    G_PARAM_READWRITE);
 	/**
 	 * SoupMessage:is-top-level-navigation:
 	 *
 	 * Set when the message is navigating between top level domains.
 	 *
 	 */
-	g_object_class_install_property (
-		object_class, PROP_IS_TOP_LEVEL_NAVIGATION,
+        properties[PROP_IS_TOP_LEVEL_NAVIGATION] =
 		g_param_spec_boolean ("is-top-level-navigation",
 				     "Is top-level navigation",
 				     "If the current messsage is navigating between top-levels",
 				     FALSE,
-				     G_PARAM_READWRITE));
-	g_object_class_install_property (
-		object_class, PROP_REQUEST_HEADERS,
+				     G_PARAM_READWRITE);
+        properties[PROP_REQUEST_HEADERS] =
 		g_param_spec_boxed ("request-headers",
 				    "Request Headers",
 				    "The HTTP request headers",
 				    SOUP_TYPE_MESSAGE_HEADERS,
 				    G_PARAM_READABLE |
-				    G_PARAM_STATIC_STRINGS));
-	g_object_class_install_property (
-		object_class, PROP_RESPONSE_HEADERS,
+				    G_PARAM_STATIC_STRINGS);
+        properties[PROP_RESPONSE_HEADERS] =
 		g_param_spec_boxed ("response-headers",
 				    "Response Headers",
 				     "The HTTP response headers",
 				    SOUP_TYPE_MESSAGE_HEADERS,
 				    G_PARAM_READABLE |
-				    G_PARAM_STATIC_STRINGS));
+				    G_PARAM_STATIC_STRINGS);
 	/**
 	 * SoupMessage:tls-peer-certificate:
 	 *
 	 * The peer's #GTlsCertificate associated with the message
 	 *
 	 */
-	g_object_class_install_property (
-		object_class, PROP_TLS_PEER_CERTIFICATE,
+        properties[PROP_TLS_PEER_CERTIFICATE] =
 		g_param_spec_object ("tls-peer-certificate",
 				     "TLS Peer Certificate",
 				     "The TLS peer certificate associated with the message",
 				     G_TYPE_TLS_CERTIFICATE,
 				     G_PARAM_READABLE |
-				     G_PARAM_STATIC_STRINGS));
+				     G_PARAM_STATIC_STRINGS);
 	/**
 	 * SoupMessage:tls-peer-certificate-errors:
 	 *
 	 * The verification errors on #SoupMessage:tls-peer-certificate
 	 *
 	 */
-	g_object_class_install_property (
-		object_class, PROP_TLS_PEER_CERTIFICATE_ERRORS,
+        properties[PROP_TLS_PEER_CERTIFICATE_ERRORS] =
 		g_param_spec_flags ("tls-peer-certificate-errors",
 				    "TLS Peer Certificate Errors",
 				    "The verification errors on the message's TLS peer certificate",
 				    G_TYPE_TLS_CERTIFICATE_FLAGS, 0,
 				    G_PARAM_READABLE |
-				    G_PARAM_STATIC_STRINGS));
+				    G_PARAM_STATIC_STRINGS);
+        /**
+         * SoupMessage:remote-address:
+         *
+         * The remote #GSocketAddress of the connection associated with the message
+         *
+         */
+        properties[PROP_REMOTE_ADDRESS] =
+                g_param_spec_object ("remote-address",
+                                     "Remote Address",
+                                     "The remote address of the connection associated with the message",
+                                     G_TYPE_SOCKET_ADDRESS,
+                                     G_PARAM_READABLE |
+                                     G_PARAM_STATIC_STRINGS);
 	/**
 	 SoupMessage:priority:
 	 *
@@ -747,15 +753,14 @@ soup_message_class_init (SoupMessageClass *message_class)
 	 * soup_message_set_priority() for further details.
 	 *
 	 **/
-	g_object_class_install_property (
-		object_class, PROP_PRIORITY,
+        properties[PROP_PRIORITY] =
 		g_param_spec_enum ("priority",
 				   "Priority",
 				   "The priority of the message",
 				   SOUP_TYPE_MESSAGE_PRIORITY,
 				   SOUP_MESSAGE_PRIORITY_NORMAL,
 				   G_PARAM_READWRITE |
-				   G_PARAM_STATIC_STRINGS));
+				   G_PARAM_STATIC_STRINGS);
 
 	/**
 	 * SoupMessage:is-options-ping:
@@ -765,14 +770,15 @@ soup_message_class_init (SoupMessageClass *message_class)
          * path of #SoupMessage:uri will be ignored and
          * #SoupMessage:method set to %SOUP_METHOD_OPTIONS.
 	 */
-	g_object_class_install_property (
-		object_class, PROP_IS_OPTIONS_PING,
+        properties[PROP_IS_OPTIONS_PING] =
 		g_param_spec_boolean ("is-options-ping",
 				      "Is Options Ping",
 				      "The message is an OPTIONS ping",
                                       FALSE,
 				      G_PARAM_READWRITE |
-				      G_PARAM_STATIC_STRINGS));
+				      G_PARAM_STATIC_STRINGS);
+
+        g_object_class_install_properties (object_class, LAST_PROPERTY, properties);
 }
 
 
@@ -1305,8 +1311,22 @@ soup_message_set_tls_peer_certificate (SoupMessage         *msg,
         g_clear_object (&priv->tls_peer_certificate);
         priv->tls_peer_certificate = tls_certificate ? g_object_ref (tls_certificate) : NULL;
         priv->tls_peer_certificate_errors = tls_errors;
-        g_object_notify (G_OBJECT (msg), "tls-peer-certificate");
-        g_object_notify (G_OBJECT (msg), "tls-peer-certificate-errors");
+        g_object_notify_by_pspec (G_OBJECT (msg), properties[PROP_TLS_PEER_CERTIFICATE]);
+        g_object_notify_by_pspec (G_OBJECT (msg), properties[PROP_TLS_PEER_CERTIFICATE_ERRORS]);
+}
+
+static void
+soup_message_set_remote_address (SoupMessage    *msg,
+                                 GSocketAddress *address)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+
+        if (priv->remote_address == address)
+                return;
+
+        g_clear_object (&priv->remote_address);
+        priv->remote_address = address ? g_object_ref (address) : NULL;
+        g_object_notify_by_pspec (G_OBJECT (msg), properties[PROP_REMOTE_ADDRESS]);
 }
 
 SoupConnection *
@@ -1380,6 +1400,14 @@ re_emit_tls_certificate_changed (SoupMessage    *msg,
                                                soup_connection_get_tls_certificate_errors (conn));
 }
 
+static void
+connection_remote_address_changed (SoupMessage    *msg,
+                                   GParamSpec     *pspec,
+                                   SoupConnection *conn)
+{
+        soup_message_set_remote_address (msg, soup_connection_get_remote_address (conn));
+}
+
 void
 soup_message_set_connection (SoupMessage    *msg,
 			     SoupConnection *conn)
@@ -1391,6 +1419,7 @@ soup_message_set_connection (SoupMessage    *msg,
 
 	if (priv->connection) {
 		g_signal_handlers_disconnect_by_data (priv->connection, msg);
+                priv->io_data = NULL;
 		g_object_remove_weak_pointer (G_OBJECT (priv->connection), (gpointer*)&priv->connection);
 	}
 
@@ -1404,6 +1433,7 @@ soup_message_set_connection (SoupMessage    *msg,
         soup_message_set_tls_peer_certificate (msg,
                                                soup_connection_get_tls_certificate (priv->connection),
                                                soup_connection_get_tls_certificate_errors (priv->connection));
+        soup_message_set_remote_address (msg, soup_connection_get_remote_address (priv->connection));
 
 	g_signal_connect_object (priv->connection, "event",
 				 G_CALLBACK (re_emit_connection_event),
@@ -1414,6 +1444,9 @@ soup_message_set_connection (SoupMessage    *msg,
 	g_signal_connect_object (priv->connection, "notify::tls-certificate",
 				 G_CALLBACK (re_emit_tls_certificate_changed),
 				 msg, G_CONNECT_SWAPPED);
+        g_signal_connect_object (priv->connection, "notify::remote-address",
+                                 G_CALLBACK (connection_remote_address_changed),
+                                 msg, G_CONNECT_SWAPPED);
 }
 
 /**
@@ -1438,6 +1471,7 @@ soup_message_cleanup_response (SoupMessage *msg)
 
         if (!priv->connection) {
                 soup_message_set_tls_peer_certificate (msg, NULL, 0);
+                soup_message_set_remote_address (msg, NULL);
                 priv->last_connection_id = 0;
         }
 
@@ -1490,7 +1524,7 @@ soup_message_set_flags (SoupMessage *msg, SoupMessageFlags flags)
 		return;
 
 	priv->msg_flags = flags;
-	g_object_notify (G_OBJECT (msg), "flags");
+	g_object_notify_by_pspec (G_OBJECT (msg), properties[PROP_FLAGS]);
 }
 
 /**
@@ -1592,7 +1626,7 @@ soup_message_set_http_version (SoupMessage *msg, SoupHTTPVersion version)
 	priv->http_version = version;
 	if (priv->status_code == SOUP_STATUS_NONE)
 		priv->orig_http_version = version;
-	g_object_notify (G_OBJECT (msg), "http-version");
+	g_object_notify_by_pspec (G_OBJECT (msg), properties[PROP_HTTP_VERSION]);
 }
 
 /**
@@ -1696,7 +1730,7 @@ soup_message_set_uri (SoupMessage *msg, GUri *uri)
         }
 
 	priv->uri = normalized_uri;
-	g_object_notify (G_OBJECT (msg), "uri");
+	g_object_notify_by_pspec (G_OBJECT (msg), properties[PROP_URI]);
 }
 
 /**
@@ -1738,7 +1772,7 @@ soup_message_set_status (SoupMessage *msg,
 
         if (priv->status_code != status_code) {
                 priv->status_code = status_code;
-                g_object_notify (G_OBJECT (msg), "status-code");
+                g_object_notify_by_pspec (G_OBJECT (msg), properties[PROP_STATUS_CODE]);
         }
 
         if (reason_phrase) {
@@ -1903,7 +1937,7 @@ soup_message_set_first_party (SoupMessage *msg,
 	}
 
 	priv->first_party = g_steal_pointer (&first_party_normalized);
-	g_object_notify (G_OBJECT (msg), "first-party");
+	g_object_notify_by_pspec (G_OBJECT (msg), properties[PROP_FIRST_PARTY]);
 }
 
 /**
@@ -1967,7 +2001,7 @@ soup_message_set_site_for_cookies (SoupMessage *msg,
 	}
 
 	priv->site_for_cookies = g_steal_pointer (&site_for_cookies_normalized);
-	g_object_notify (G_OBJECT (msg), "site-for-cookies");
+	g_object_notify_by_pspec (G_OBJECT (msg), properties[PROP_SITE_FOR_COOKIES]);
 }
 
 /**
@@ -1993,7 +2027,7 @@ soup_message_set_is_top_level_navigation (SoupMessage *msg,
 		return;
 
 	priv->is_top_level_navigation = is_top_level_navigation;
-	g_object_notify (G_OBJECT (msg), "is-top-level-navigation");
+	g_object_notify_by_pspec (G_OBJECT (msg), properties[PROP_IS_TOP_LEVEL_NAVIGATION]);
 }
 
 /**
@@ -2111,7 +2145,7 @@ soup_message_set_priority (SoupMessage        *msg,
                 return;
 
         priv->priority = priority;
-	g_object_notify (G_OBJECT (msg), "priority");
+	g_object_notify_by_pspec (G_OBJECT (msg), properties[PROP_PRIORITY]);
 }
 
 /**
@@ -2136,7 +2170,7 @@ soup_message_get_priority (SoupMessage *msg)
 	return priv->priority;
 }
 
-SoupClientMessageIOData *
+SoupClientMessageIO *
 soup_message_get_io_data (SoupMessage *msg)
 {
 	SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
@@ -2145,13 +2179,131 @@ soup_message_get_io_data (SoupMessage *msg)
 }
 
 void
-soup_message_set_io_data (SoupMessage             *msg,
-			  SoupClientMessageIOData *io)
+soup_message_io_finished (SoupMessage *msg)
 {
-	SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
 
-	soup_client_message_io_data_free (priv->io_data);
-	priv->io_data = io;
+        if (!priv->io_data)
+                return;
+
+        g_assert (priv->connection != NULL);
+        soup_client_message_io_finished (g_steal_pointer (&priv->io_data));
+}
+
+void
+soup_message_io_pause (SoupMessage *msg)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+
+        g_return_if_fail (priv->io_data != NULL);
+
+        soup_client_message_io_pause (priv->io_data);
+}
+
+void
+soup_message_io_unpause (SoupMessage *msg)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+
+        g_return_if_fail (priv->io_data != NULL);
+
+        soup_client_message_io_unpause (priv->io_data);
+}
+
+gboolean
+soup_message_is_io_paused (SoupMessage *msg)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+
+        return priv->io_data && soup_client_message_io_is_paused (priv->io_data);
+}
+
+/**
+ * soup_message_io_in_progress:
+ * @msg: a #SoupMessage
+ *
+ * Tests whether or not I/O is currently in progress on @msg.
+ *
+ * Returns: whether or not I/O is currently in progress.
+ **/
+gboolean
+soup_message_io_in_progress (SoupMessage *msg)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+
+        return priv->io_data != NULL;
+}
+
+void
+soup_message_io_run (SoupMessage *msg,
+                     gboolean     blocking)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+
+        soup_client_message_io_run (priv->io_data, blocking);
+}
+
+gboolean
+soup_message_io_run_until_read (SoupMessage  *msg,
+                                GCancellable *cancellable,
+                                GError      **error)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+
+        return soup_client_message_io_run_until_read (priv->io_data, cancellable, error);
+}
+
+void
+soup_message_io_run_until_read_async (SoupMessage        *msg,
+                                      int                 io_priority,
+                                      GCancellable       *cancellable,
+                                      GAsyncReadyCallback callback,
+                                      gpointer            user_data)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+
+        soup_client_message_io_run_until_read_async (priv->io_data, io_priority, cancellable, callback, user_data);
+}
+
+gboolean
+soup_message_io_run_until_read_finish (SoupMessage  *msg,
+                                       GAsyncResult *result,
+                                       GError      **error)
+{
+        return g_task_propagate_boolean (G_TASK (result), error);
+}
+
+gboolean
+soup_message_io_run_until_finish (SoupMessage  *msg,
+                                  gboolean      blocking,
+                                  GCancellable *cancellable,
+                                  GError      **error)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+
+        return soup_client_message_io_run_until_finish (priv->io_data, blocking, cancellable, error);
+}
+
+void
+soup_message_send_item (SoupMessage              *msg,
+                        SoupMessageQueueItem     *item,
+                        SoupMessageIOCompletionFn completion_cb,
+                        gpointer                  user_data)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+
+        priv->io_data = soup_connection_setup_message_io (priv->connection, msg);
+        soup_client_message_io_send_item (priv->io_data, item,
+                                          completion_cb, user_data);
+}
+
+GInputStream *
+soup_message_io_get_response_istream (SoupMessage  *msg,
+                                      GError      **error)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+
+        return soup_client_message_io_get_response_stream (priv->io_data, error);
 }
 
 SoupContentSniffer *
@@ -2289,7 +2441,7 @@ soup_message_set_reason_phrase (SoupMessage *msg, const char *reason_phrase)
 
         g_free (priv->reason_phrase);
         priv->reason_phrase = g_strdup (reason_phrase);
-        g_object_notify (G_OBJECT (msg), "reason-phrase");
+        g_object_notify_by_pspec (G_OBJECT (msg), properties[PROP_REASON_PHRASE]);
 }
 
 void
@@ -2303,7 +2455,7 @@ soup_message_set_method (SoupMessage *msg,
                 return;
 
         priv->method = new_method;
-        g_object_notify (G_OBJECT (msg), "method");
+        g_object_notify_by_pspec (G_OBJECT (msg), properties[PROP_METHOD]);
 }
 
 /**
@@ -2348,7 +2500,7 @@ soup_message_set_is_options_ping (SoupMessage *msg,
                 return;
 
         priv->is_options_ping = is_options_ping;
-        g_object_notify (G_OBJECT (msg), "is-options-ping");
+        g_object_notify_by_pspec (G_OBJECT (msg), properties[PROP_IS_OPTIONS_PING]);
         if (priv->is_options_ping)
                 soup_message_set_method (msg, SOUP_METHOD_OPTIONS);
 }
@@ -2373,6 +2525,31 @@ soup_message_get_connection_id (SoupMessage *msg)
         return priv->last_connection_id;
 }
 
+/**
+ * soup_message_get_remote_address:
+ * @msg: The #SoupMessage
+ *
+ * Get the remote #GSocketAddress of the connection associated with the message.
+ * The returned address can be %NULL if the connection hasn't been established yet,
+ * or the resource was loaded from the disk cache.
+ * In case of proxy connections, the remote address returned is a #GProxyAddress.
+ * If #SoupSession::remote-connetable is set the returned address id for the connection
+ * ot the session's remote connectable.
+ *
+ * Returns: (transfer none) (nullable): a #GSocketAddress or %NULL if the connection
+ *     hasn't been established
+ */
+GSocketAddress *
+soup_message_get_remote_address (SoupMessage *msg)
+{
+        SoupMessagePrivate *priv;
+
+        g_return_val_if_fail (SOUP_IS_MESSAGE (msg), NULL);
+
+        priv = soup_message_get_instance_private (msg);
+        return priv->remote_address;
+}
+
 /**
  * soup_message_get_metrics:
  * @msg: The #SoupMessage
