@@ -935,6 +935,8 @@ do_one_connection_event_test (SoupSession *session,
 	SoupMessage *msg;
 	GBytes *body;
         SoupMessageMetrics *metrics;
+        GSocketAddress *remote_address;
+        char *ip_address;
 
 	msg = soup_message_new ("GET", uri);
         if (collect_metrics) {
@@ -952,6 +954,7 @@ do_one_connection_event_test (SoupSession *session,
 	g_signal_connect (msg, "network-event",
 			  G_CALLBACK (network_event),
 			  &events);
+        g_assert_null (soup_message_get_remote_address (msg));
 	body = soup_session_send_and_read (session, msg, NULL, NULL);
 	soup_test_assert_message_status (msg, SOUP_STATUS_OK);
 	while (*events) {
@@ -979,6 +982,19 @@ do_one_connection_event_test (SoupSession *session,
                 g_assert_null (metrics);
         }
 
+        remote_address = soup_message_get_remote_address (msg);
+        g_assert_true (G_IS_INET_SOCKET_ADDRESS (remote_address));
+        ip_address = g_inet_address_to_string (g_inet_socket_address_get_address (G_INET_SOCKET_ADDRESS (remote_address)));
+        g_assert_cmpstr (ip_address, ==, "127.0.0.1");
+        g_free (ip_address);
+        if (G_IS_PROXY_ADDRESS (remote_address)) {
+                g_assert_cmpuint (g_inet_socket_address_get_port (G_INET_SOCKET_ADDRESS (remote_address)), ==, 47526);
+                g_assert_cmpuint (g_proxy_address_get_destination_port (G_PROXY_ADDRESS (remote_address)), ==, g_uri_get_port (soup_message_get_uri (msg)));
+                g_assert_cmpstr (g_proxy_address_get_destination_hostname (G_PROXY_ADDRESS (remote_address)), ==, "127.0.0.1");
+        } else {
+                g_assert_cmpuint (g_inet_socket_address_get_port (G_INET_SOCKET_ADDRESS (remote_address)), ==, g_uri_get_port (soup_message_get_uri (msg)));
+        }
+
 	g_bytes_unref (body);
 	g_object_unref (msg);
 	soup_session_abort (session);
@@ -1052,6 +1068,12 @@ do_one_connection_event_fail_test (SoupSession *session,
                 g_assert_null (metrics);
         }
 
+        /* When failing the TLS handshake we got a remote address */
+        if (g_str_equal (uri, HTTPS_SERVER))
+                g_assert_nonnull (soup_message_get_remote_address (msg));
+        else
+                g_assert_null (soup_message_get_remote_address (msg));
+
         g_bytes_unref (body);
         g_object_unref (msg);
         soup_session_abort (session);
@@ -1401,6 +1423,119 @@ do_steal_connection_preconnect_fail_test (const char *uri,
         soup_test_session_abort_unref (session);
 }
 
+typedef struct {
+        GMainLoop *loop;
+        guint count;
+} StealPreconnectTestData;
+
+static void
+steal_preconnect_finished (SoupSession             *session,
+                           GAsyncResult            *result,
+                           StealPreconnectTestData *data)
+{
+        soup_session_preconnect_finish (session, result, NULL);
+        if (++data->count == 2)
+                g_main_loop_quit (data->loop);
+}
+
+static void
+steal_preconnect_send_and_read_finished (SoupSession             *session,
+                                         GAsyncResult            *result,
+                                         StealPreconnectTestData *data)
+{
+        GBytes *bytes;
+
+        bytes = soup_session_send_and_read_finish (session, result, NULL);
+        g_bytes_unref (bytes);
+        if (++data->count == 2)
+                g_main_loop_quit (data->loop);
+}
+
+static void
+do_steal_preconnect_connection_test (void)
+{
+        SoupSession *session;
+        SoupMessage *msg;
+        StealPreconnectTestData data = { NULL, 0 };
+        PreconnectTestData data1 = { NULL, NULL, "rRcCx", SOUP_CONNECTION_DISCONNECTED, NULL, FALSE };
+        PreconnectTestData data2 = { NULL, NULL, "rRcCx", SOUP_CONNECTION_DISCONNECTED, NULL, FALSE };
+
+        session = soup_test_session_new (NULL);
+
+        /* Preconnect requests should never steal the connection of another preconnect. */
+
+        data.loop = g_main_loop_new (NULL, FALSE);
+        msg = soup_message_new ("HEAD", HTTP_SERVER);
+        g_signal_connect (msg, "network-event",
+                          G_CALLBACK (preconnection_test_message_network_event),
+                          &data1);
+        soup_session_preconnect_async (session, msg, G_PRIORITY_DEFAULT, NULL,
+                                       (GAsyncReadyCallback)steal_preconnect_finished,
+                                       &data);
+        g_object_unref (msg);
+
+        msg = soup_message_new ("HEAD", HTTP_SERVER);
+        g_signal_connect (msg, "network-event",
+                          G_CALLBACK (preconnection_test_message_network_event),
+                          &data2);
+        soup_session_preconnect_async (session, msg, G_PRIORITY_DEFAULT, NULL,
+                                       (GAsyncReadyCallback)steal_preconnect_finished,
+                                       &data);
+        g_object_unref (msg);
+
+        g_main_loop_run (data.loop);
+        g_assert_nonnull (data1.conn);
+        g_assert_nonnull (data2.conn);
+        g_assert_false (data1.conn == data2.conn);
+        g_assert_cmpint (data1.state, ==, SOUP_CONNECTION_IDLE);
+        g_assert_cmpint (data2.state, ==, SOUP_CONNECTION_IDLE);
+
+        while (*data1.events) {
+                soup_test_assert (!*data1.events,
+                                  "Expected %s",
+                                  event_name_from_abbrev (*data1.events));
+                soup_test_assert (!*data2.events,
+                                  "Expected %s",
+                                  event_name_from_abbrev (*data2.events));
+                data1.events++;
+                data2.events++;
+        }
+
+        data.count = 0;
+        g_clear_object (&data1.conn);
+        g_clear_object (&data2.conn);
+
+        msg = soup_message_new ("GET", HTTP_SERVER);
+        g_signal_connect (msg, "network-event",
+                          G_CALLBACK (preconnection_test_message_network_event),
+                          &data1);
+        soup_session_send_and_read_async (session, msg, G_PRIORITY_DEFAULT, NULL,
+                                          (GAsyncReadyCallback)steal_preconnect_send_and_read_finished,
+                                          &data);
+        g_object_unref (msg);
+
+        msg = soup_message_new ("GET", HTTP_SERVER);
+        g_signal_connect (msg, "network-event",
+                          G_CALLBACK (preconnection_test_message_network_event),
+                          &data2);
+        soup_session_send_and_read_async (session, msg, G_PRIORITY_DEFAULT, NULL,
+                                          (GAsyncReadyCallback)steal_preconnect_send_and_read_finished,
+                                          &data);
+        g_object_unref (msg);
+
+        g_main_loop_run (data.loop);
+
+        /* connection-created hasn't been called. */
+        g_assert_null (data1.conn);
+        g_assert_null (data2.conn);
+        g_assert_cmpint (data1.state, ==, SOUP_CONNECTION_IDLE);
+        g_assert_cmpint (data2.state, ==, SOUP_CONNECTION_IDLE);
+
+        g_main_loop_unref (data.loop);
+
+        soup_test_session_abort_unref (session);
+}
+
 static void
 do_connection_preconnect_test (void)
 {
@@ -1440,6 +1575,9 @@ do_connection_preconnect_test (void)
                                                           "r");
         } else
                 debug_printf (1, "    https -- SKIPPING\n");
+
+        debug_printf (1, "    preconnect should never steal a connection\n");
+        do_steal_preconnect_connection_test ();
 }
 
 static void
